/* checkers.h - Checkers game module								*/
/* This module will perform the actions needed in a checkers game	*/

#ifndef	CHECKERS_H
#define	CHECKERS_H

#include <stdio.h>

#define	BOARD_SIZE			8									/* Defines the number of squares in each row and each column. Must be an even number.	*/
#define	STARTING_NUM_ROWS	3									/* Defines the number of rows with pieces a player starts with.							*/
#define	MAX_PIECES			(BOARD_SIZE/2)*STARTING_NUM_ROWS	/* Defines the maximum number number of pieces a player starts with.					*/	
#define	COLOR_NAME_SIZE		5									/* Defines the length of the colors' names.												*/

typedef	enum			{black = 1, white = (-1), empty = 0}	p_color;		/* Defines possible colors.	*/
typedef	enum			{human = 1, computer = 0, file = (-1)}	p_user;			/* Defines possible users.	*/
typedef	enum			{move = 0, jump = 1}					m_type;			/* Defines type of move.	*/
typedef	struct node*	list;													/* Defines a list.			*/

typedef	struct location_t*	location;	/* Defines a point on the board.	*/
typedef	struct movement_t*	movement;	/* Defines a movement of one piece.	*/
typedef	struct player_t*	player;		/* Defines a player.				*/
typedef	struct piece_t*		piece;		/* Defines a piece.					*/
typedef	struct board_t*		board;		/* Defines a playing board.			*/
typedef	struct gamelog_t*	gamelog;	/* Defines a game log.				*/

/*-----------------------*/
/* ~ General Functions ~ */
/*------------------------------------------------------*/
/* Sves a game log to a file.							*/
/* Parameters: Log to save, name of file to be saved.	*/
/* Return value: 1 - Saved, 0 - Not saved.				*/
/*------------------------------------------------------*/
int		save_log(gamelog log, char* filename);

/*----------------------------------------------*/
/* Creates a player.							*/
/* Parameters: None.							*/
/* Return value: New player. NULL if failed.	*/
/*----------------------------------------------*/
player	create_player();

/*--------------------------------------------------------------*/
/* Creates a board. ' ' - White squares, ':' - Black squares.	*/
/* Parameters: None.											*/
/* Return value: New player. NULL if failed.					*/
/*--------------------------------------------------------------*/
board	create_board();

/*----------------------------------------------*/
/* Creates a game log.							*/
/* Parameters: None.							*/
/* Return value: New player. NULL if failed.	*/
/*----------------------------------------------*/
gamelog	create_log();

/*----------------------------------------------------------*/
/* Gets a player and defines the type of user playing.		*/
/* Parameters: Player, type of user: Human, computer, file.	*/
/* Return value: None.										*/
/*----------------------------------------------------------*/
void	set_player(player pl, p_user u);

/*----------------------------------------------------------------------*/
/* Gets a board and a number and defines the number as dificulty level.	*/
/* Parameters: Board, number.											*/
/* Return value: None.													*/
/* Be advised: dificulty uses memory exponentially! Handle with care!	*/
/*----------------------------------------------------------------------*/
void	set_dificulty(board brd, int dificulty);

/*--------------------------------------------------------------*/
/* Gets a player and frees all dinamically allocated memory.	*/
/* Parameters: Player.											*/
/* Return value: None.											*/
/*--------------------------------------------------------------*/
void	destroy_player(player pl);

/*----------------------------------------------------------*/
/* Gets a board and frees all dinamically allocated memory.	*/
/* Parameters: Board.										*/
/* Return value: None.										*/
/*----------------------------------------------------------*/
void	destroy_board(board brd);

/*--------------------------------------------------------------*/
/* Gets a game log and frees all dinamically allocated memory.	*/
/* Parameters: Game log.										*/
/* Return value: None.											*/
/*--------------------------------------------------------------*/
void	destroy_log(gamelog log);

/*--------------------------*/
/* ~ Game State Functions ~ */
/*------------------------------------------------------------------*/
/* Gets a board, two players and a turn tracker. Initializes all.	*/
/* Parameters: Board, player 1, player 2, game log.					*/
/* Return value: None.												*/
/*------------------------------------------------------------------*/
void	init(board brd, player p1, player p2);

/*----------------------------------------------*/
/* Gets a board And prints it to the screen.	*/
/* Parameters: Board.							*/
/* Return value: None.							*/
/*----------------------------------------------*/
void	print_board(board brd);

/*----------------------------------------------*/
/* Gets a game log and prints it to the screen.	*/
/* Parameters: Game log.						*/
/* Return value: None.							*/
/*----------------------------------------------*/
void	print_log(gamelog log);

/*--------------------------------------------------------------------------------------------------------------------------*/
/* Gets a board and plays one turn.																							*/
/* Parameters: Board, game log, name of file.																				*/
/* Return value:																											*/
/* 1 - p1 wins, -1 - p2 wins, 0 - no desicion.																				*/
/* >90 - Special action:		100 - End of file.																			*/
/* 99 - Can't open file; 98 - Template error; 97 - Illegal move; 96 - Overflow; 95 - Play instructions after game ended.	*/
/* 94 - Game ended at user request by typing 'Q';									91 - Print board; 92 - Print log.		*/
/*--------------------------------------------------------------------------------------------------------------------------*/
int		turn(board brd, gamelog log, char* filename);

#endif