#include <stdio.h>
#include "str.h"
#include "checkers.h"

int		menu();							/* Runs the menu and returns the users choice.							 						*/
										/* 1 - Two players; 2 - Black against computer; 3 - White against computer; 4 - Computer alone.	*/
										/* 5 - Simulation from file; 6 - Save log to file; 0 to terminate the program.					*/
int		choose_dificulty(board brd);	/* Ask the player for dificulty level when playing against the computer.						*/
void	print_instructions();			/* Prints the instructions.																		*/

int main(int argc, char* argv[]){

	int		choice, win, played = 0, saved = 0;
	player	black_p, white_p;
	board	brd;
	gamelog	log;	

	if (argc != 2){
		printf("Illegal number of arguments. Program terminated.\n\n");
		return 1;
	}

	system("clear");		/* Clear screen. */
	printf("Welcome to Checkers!\n\n");
	print_instructions();
	system("clear");		/* Clear screen. */

	/* Creates a board, two players and a log archive. */
	black_p = create_player();
	white_p = create_player();
	brd = create_board();
	log = create_log();

	if (!black_p || !white_p || !brd || !log){
		printf("Not enough memory. Program is shutting down!\n\n");
		return 1;
	}

	/* While within this loop, the player is in the program. */
	while (choice = menu()){

		init(brd, black_p, white_p);				/* I nitializes the board. */

		switch (choice){
			case 1:{	/* Two players' game - Both players are human. */
				played = 1;
				set_player(black_p, human);
				set_player(white_p, human);
				break;
				   }
			case 2:{	/* Black is human, white is played by the computer. */
				played = 1;
				set_player(black_p, human);
				set_player(white_p, computer);
				choose_dificulty(brd);
				break;
				   }
			case 3:{	/* White is human, black is played by the computer. */
				played = 1;
				set_player(black_p, computer);
				set_player(white_p, human);
				choose_dificulty(brd);
				break;
				   }
			case 4:{	/* Both sides are played by the computer. */
				played = 1;
				set_player(black_p, computer);
				set_player(white_p, computer);
				choose_dificulty(brd);
				break;
				   }
			case 5:{	/* Both sides are played by the computer, as it appears in a file. */
				played = 0;
				set_player(black_p, file);
				set_player(white_p, file);
				break;
				   }
			case 6:{	/* Saves a log into a file. */
				saved = save_log(log, argv[1]);
				if (saved){
					played = 0;			/* No games wre played without saving. */
					system("clear");		/* Clear screen. */
					printf("Saved successfully.\n");
				}
				continue;							/* Return to menue without playing a game.	*/
				   }
			case 7:{	/* Prints the board and a game log. */
				played = 0;							/* No games wre played without saving. */
				system("clear");		/* Clear screen. */
				print_log(log);
				continue;							/* Return to menue without playing a game.	*/
				   }
		}

		system("clear");		/* Clear screen. */
		destroy_log(log);		/* Resets the game log. */
		log = create_log();

		do{										/* While in this loop, a game is being conducted. */

			win = turn(brd, log, argv[1]);		/* One turn is played and an outcome is returned. */

			if (win > 90){						/* An error has ocurred wile using a file. */
				switch (win){
					case 100:{			/* Reached EOF. */
						printf("File ended. Simulation terminated.\n");
						break;
							 }
					case  99:{			/* Can't open file for reading. */
						printf("File cannot be opened. Game terminated.\n");
						break;
							 }
					case  98:{			/* File contains a command not recognized. */
						printf("An illegal instruction was detected. Game terminated.\n");
						break;
							 }
					case  97:{			/* File contains a move that violate the rules of the game. */
						printf("An illegal move was detected. Game terminated.\n");
						break;
							 }
					case  96:{			/* File contains a move that overflows or underflows the board. */
						printf("A piece was moved beyond the board's limits. Game terminated.\n");
						break;
							 }
					case  95:{			/* File contains moves after a game was concluded. */
						printf("You can't play a game that already ended.\n");
						break;
							 }
					case  94:{			/* Player typed 'Q' to end the game. */
						printf("Game concluded at player's request.\n");
						getchar();
						break;
							 }
					case  91:{			/* File contains instruction to print current state of the board. */
						print_board(brd);
						win = 0;		/* Resets gamestate to allow the game to continue. */
						continue;
							 }
					case  92:{			/* File contains instruction to print all moves made from the beginning of the file. */
						print_log(log);
						win = 0;		/* Resets gamestate to allow the game to continue. */
						continue;
							 }
				}

				printf("Press ENTER to continue...\n");
				FLUSH_INPUT;
				break;					/* If the file returns an error - Stop the game. */
			}

		} while (!win);					/* Runs while no winner is found. */
		system("clear");		/* Clear screen. */
		print_board(brd);

		if (win > 90){					/* IF the game ended because of an error - Go back to menue. */
			system("clear");		/* Clear screen. */
			continue;
		}

		if (win == black){				/* Prints the winner. */
			printf("BLACK WINS!!!\n");
		}
		if (win == white){
			printf("WHITE WINS!!!\n");
		}
		printf("Press ENTER to continue...\n");
		FLUSH_INPUT;

		if (choice == 4){			/* If automatic game log was chosen, ask to save. */
			saved = save_log(log, argv[1]);
			if (saved){
				played = 0;			/* No games wre played without saving. */
				system("clear");		/* Clear screen. */
				printf("Saved successfully.\n");
			}
		}

		system("clear");		/* Clear screen. */
		saved = 0;

	}

	if (played && !saved){				/* Checks for unsaved log. */
		do{

			printf("The las game log wasn't saved. Would you like to save it?\n");
			printf("1. Yes.\n");
			printf("2. No.\n");
			printf("Your choice: ");
			choice = getchar();
			FLUSH_INPUT;
			printf("\n");

			switch (choice){
			case '1':{
				saved = save_log(log, argv[1]);
				if (saved){
					printf("Saved successfully.\n");
				}
				break;
					 }
			case '2':{
				printf("Operation Cancelled.\n");
				break;
					 }
			default:{
				system("clear");		/* Clear screen. */
				printf("Your choice was illegal.\n");
					}
			}

		} while ((choice != '1') && (choice != '2'));
	}

	/* Clear all memory allocated dinamically. */
	destroy_player(black_p);
	destroy_player(white_p);
	destroy_board(brd);
	destroy_log(log);

	printf("Thank you for playing!\n");
	printf("Have a nice day...\n\n");

	return 0;
} /*main*/

int		menu(){

	char	choice;

	do{		/* Run the menue while no legal choice is made. */

		system("clear");		/* Clear screen. */
		printf("Type '?' to review the instructions.\n\n");
		printf("Please choose one of the following options:\n");
		printf("1. Two players game.\n");
		printf("2. One player game playing Black.\n");
		printf("3. One player game playing White.\n");
		printf("4. Create automatic game log.\n");
		printf("5. Execute simulation from a file.\n");
		printf("6. Save game log to file.\n");
		printf("7. Print last game log.\n");
		printf("8. Exit.\n");
		printf("Your choice: ");
		choice = getchar();
		FLUSH_INPUT;
		printf("\n");

		switch (choice){
			case '1': return 1;
			case '2': return 2;
			case '3': return 3;
			case '4': return 4;
			case '5': return 5;
			case '6': return 6;
			case '7': return 7;
			case '8': choice = 'Q';
			case 'q': choice = 'Q';
			case 'Q': break;	/* 8, q and Q terminate the program. */
			case '?':{
				print_instructions();
				break;
					 }
			default:{
				system("clear");		/* Clear screen. */
				printf("Your choice was illegal.\n");
					}
		}

	} while (choice != 'Q');

	return 0;

} /*menu*/

int		choose_dificulty(board brd){

	char choice;

	do{

		printf("Please choose dificulty level:\n");
			printf("1. Easy.\n");
			printf("2. Medium.\n");
			printf("3. Hard.\n");
			printf("Your choice: ");
			choice = getchar();
			FLUSH_INPUT;
			printf("\n");

			switch (choice){
			case '1':
			case '2':
			case '3':{
				set_dificulty(brd, (choice + 1 - '0'));
				break;
					 }
			default :{
				system("clear");		/* Clear screen. */
				printf("Your choice was illegal.\n");
					 }
			}

		} while ((choice < '1') || (choice > '3'));

} /*choose_dificulty*/

void	print_instructions(){

	printf("Instructions:\n");
	printf("Game pieces can only be moved diagonally on black squares (Marked by a colon).\n");
	printf("One player plays 'Black', the other plays 'White'. The player playing 'Black'\nmoves first.\n");
	printf("A regular move consists of moving one of your pieces diagonally one square\ntowards your opponent's side.\n");
	printf("A 'Jump' can occur if one of your opponents' pieces is immediately next to one\nof your own, and the following square is empty.\n");
	printf("You can only jump towards your opponent's side. Once a jump has been made, the\nsame piece can jump again if able to all directions.\n");
	printf("If a player can jump, he or she must do so. In case of several 'Jump' options,\nhe or she must choose one.\n");
	printf("The piece of the opponent which was jumped over is removed from the board.\n");
	printf("A player with no pieces on the board loses the game.\n");
	printf("If a move or a jump ends in the last row on the opponent's side, the piece is\nmade 'King'.\n");
	printf("A king can move to all directions as many squares as desired until reaching an\nopponent's piece.\n");
	printf("A king can jump over pieces not immediately next to it, but it will land\nimmediately after the jumped piece.\n");
	printf("If a player has no legal moves - He or she loses the game.\n");
	printf("\n");
	printf("Locations are entered in the form of [LETTER][NUMBER], representing a row\nfollowed by a column.\n");
	printf("Example: A1.\n\n");
	printf("You may type 'Q' at any time to end a game.\n\n");
	printf("Press ENTER to continue...\n");
	FLUSH_INPUT;
	system("clear");		/* Clear screen. */

} /*print_instructions*/