#include	<stdio.h>
#include	<stdlib.h>
#include	"str.h"

/* Input Functions */
char*	get_str_f(FILE* input){

	unsigned int	i, size = DEFAULT_STR_SIZE;
	char			*str;

	str = (char*)malloc(size);
	if (!str){											/* Checks allocation success */
		return NULL;
	}

	for (i = 0; (str[i] = getc(input)) != '\n'; i++){
		if (i == (size-1)){
			size += DEFAULT_STR_INC;
			str = (char*)realloc(str, size);
			if (!str){									/* Checks reallocation success */
				return NULL;
			}
		}
	}
	str[i] = '\0';

	if (*str == '\0'){									/* In case of an empty string */
		str = "N/A.";
	}

	str = (char*)realloc(str, str_length(str)+1);		/* Frees any excessive memory */

	return str;

} /*get_str_f*/

int		get_int_f(FILE* input){

	char*	temp_int;
	int		i, ok, num = 0;

	do{																	/* Gets a number and checks if legal */
		ok = 1;
		temp_int = get_str_f(input);
		if (!temp_int){
			return 0;
		}
		if (!str_compare(temp_int, "N/A.")){							/* In case of an empty string changed to "N/A." by get_str_f */
			return 0;
		}

		for (i = 0; temp_int[i] != '\0'; i++){							/* Checks the string for non-digit chars */
			if (!is_digit(temp_int[i])){
				ok = 0;
				printf("\nAn integer may comtain only digits.\n");
				printf("Please enter again:\n");
				break;
			}
		}
	} while (!ok);

	num = str_to_int(temp_int);											/* Converts the input from str to int */

	return num;

} /*get_int_f*/

float	get_float_f(FILE* input){

	char*	temp_float;
	int		i, ok, dots = 0;
	float	num = 0, multiplier;

	do{																		/* Gets a number and checks if legal */
		ok = 1;
		temp_float = get_str_f(input);
		if (!temp_float){
			return 0;
		}
		if (!str_compare(temp_float, "N/A.")){								/* In case of an empty string changed to "N/A." by get_str_f */
			return 0;
		}

		dots = 0;															/* Checks the string for non-digit chars and a one-dot limitation */
		for (i = 0; temp_float[i] != '\0'; i++){
			if ((temp_float[i] == '.') && (dots < 1)){
				dots++;
				continue;
			}
			if (!is_digit(temp_float[i])){
				ok = 0;
				printf("\nA float may contain only digits and one dot.\n");
				printf("Please enter again:\n");
				break;
			}
		}
	
	} while (!ok);

	for (i = 0; (temp_float[i] != '.') && (temp_float[i] != '\0'); i++){	/* Converts the whole part from str to float */
		num *= 10;
		num += (temp_float[i] - '0');
	}
	if (temp_float[i] == '.'){
		i++;
	}

	for (multiplier = 1; temp_float[i] != '\0'; i++){						/* Converts the decimal part from str to float */
		multiplier /= 10;
		num += (temp_float[i]-48)*multiplier;
	}

	return num;

} /*get_float_f*/

double	get_double_f(FILE* input){

	char*	temp_double;
	int		i, ok, dots = 0;
	double	num = 0, multiplier;

	do{																		/* Gets a number and checks if legal */
		ok = 1;
		temp_double = get_str_f(input);
		if (!temp_double){
			return 0;
		}
		if (!str_compare(temp_double, "N/A.")){								/* In case of an empty string changed to "N/A." by get_str_f */
			return 0;
		}

		dots = 0;															/* Checks the string for non-digit chars and a one-dot limitation */
		for (i = 0; temp_double[i] != '\0'; i++){
			if ((temp_double[i] == '.') && (dots < 1)){
				dots++;
				continue;
			}
			if (!is_digit(temp_double[i])){
				ok = 0;
				printf("\nA double may contain only digits and one dot.\n");
				printf("Please enter again:\n");
				break;
			}
		}
	
	} while (!ok);

	for (i = 0; (temp_double[i] != '.') && (temp_double[i] != '\0'); i++){	/* Converts the whole part from str to double */
		num *= 10;
		num += (temp_double[i] - '0');
	}
	if (temp_double[i] == '.'){
		i++;
	}

	for (multiplier = 1; temp_double[i] != '\0'; i++){						/* Converts the decimal part from str to double */
		multiplier /= 10;
		num += (temp_double[i]-48)*multiplier;
	}

	return num;

} /*get_double_f*/

/* Character Functions */
int		is_digit(char c){

	return ((c >= '0') && (c <= '9'));

} /*is_digit*/

int		is_letter(char c){

	upper_char(c);

	return ((c >= 'A') && (c <= 'Z'));

} /*is_letter*/

char	lower_char(char c){

	if ((c >= 'A') && (c <= 'Z')){
		c += ('a' - 'A');
	}

	return c;

} /*lower_char*/

char	upper_char(char c){

	if ((c >= 'a') && (c <= 'z')){
		c += ('A' - 'a');
	}

	return c;

} /*upper_char*/

/* Single-String Functions */
int		str_length(char* str){

	int	i;

	for (i = 0; str[i] != '\0'; i++);

	return i;

} /*str_length*/

int		str_to_int(char* str){

	int	i, num = 0;

	for (i = 0; str[i] != '\0'; i++){
		if ((str[i] < '0') || (str[i] > '9')){
			return 0;
		}

		num *= 10;
		num += (str[i] - '0');
	}

	return num;

} /*str_to_int*/

char*	int_to_str(int num){

	int		i, _num = 0, counter = 0;
	char*	str;

	while (num > 0){		/* Reverses the digits' order and counts the number of digits */
		_num *= 10;
		_num += (num%10);
		num /= 10;
		counter++;
	}

	str = (char*)malloc(counter + 1);
	if (!str){
		return NULL;
	}

	for (i = 0; i <= counter; i++){
		str[i] = ((_num%10) + '0');
	}

	return str;

} /*int_to_str*/

void	lower_str(char* str){

	int	i;

	for (i = 0; str[i] != '\0'; i++){
		str[i] = lower_char(str[i]);
	}

} /*lower_str*/

void	upper_str(char* str){

	int	i;

	for (i = 0; str[i] != '\0'; i++){
		str[i] = upper_char(str[i]);
	}

} /*upper_str*/

void	capitalize(char* str){

	str[0] = upper_char(str[0]);
	lower_str(str + 1);

} /*capitalize*/

void	str_copy(char* dest, char* source){

	int	i;

	for (i = 0; source[i] != '\0'; i++){
		dest[i] = source[i];
	}
	dest[i] = '\0';

} /*str_copy*/

/* Two-Strings Functions */
int		substring(char *str1, char *str2){

	int	i, j;

	for (i = 0; str1[i] != '\0'; i++){
		for (j = 0; (str1[i+j] == str2[j]) && (str2[j] != '\0'); j++);	/* i+j is used to preserve i's location in str1 */
		if (str2[j] == '\0'){											/* If str[j] = '\0', no differences between the strings were found */
			return 1;
		}
	}

	return 0;															/* str1 ended with no substrings */

} /*substring*/

int		str_compare(char* str1, char* str2){

	unsigned int	i, length;

	length = str_length(str1);
	for (i = 0; i <= length; i++){			/* Compares the strings. Checks the '\0' in case of a shorter string that's equal to the begining of a longer string */
		if (str1[i] > str2[i]){				/* Returns 1 if temp1 is greater */
			return 1;
		}

		if (str1[i] < str2[i]){				/* Returns -1 if temp2 is greater */
			return -1;
		}
	}

	return 0;								/* Returns 0 if equal */

} /*str_compare*/

char*	str_connect(char* str1, char* str2){

	unsigned int	i, j;
	char			*new_str;

	new_str = (char*)malloc(str_length(str1)+str_length(str2)+2);	/* Length of the two strings, one space, one '\0' */
	if (!new_str){														/* Checks allocation success */
		return NULL;
	}

	for (i = 0; str1[i] != '\0'; i++){
		new_str[i] = str1[i];
	}
	new_str[i++] = ' ';												/* Changes the '\0' to a space between the two strings */

	for (j = 0; str2[j] != '\0'; j++){
		new_str[i+j] = str2[j];
	}
	new_str[i+j] = '\0';

	return new_str;

} /*str_connect*/