/* str.h - String manipulation module																	*/
/* This header will contain string management functions, replacing string.h with personalized functions */

#ifndef	STR_H
#define	STR_H

#include	<stdio.h>
#include	<stdlib.h>

#define	DEFAULT_STR_SIZE	20
#define	DEFAULT_STR_INC		10
#define	FLUSH_INPUT			{int ch; while ((ch = getchar()) != '\n' && ch != EOF);}	/* Flushes all data remaining in the input buffer */

/* Function Shortening Definitions */
#define	get_str()		get_str_f(stdin)
#define	get_int()		get_int_f(stdin)
#define	get_float()		get_float_f(stdin)
#define	get_double()	get_double_f(stdin)

/* Input Functions */
char*	get_str_f(FILE* input);					/* Gets a string from a file. Returns NULL if failed */
int		get_int_f(FILE* input);					/* Gets an integer from a file. Returns 0 if failed */
float	get_float_f(FILE* input);				/* Gets a float from a file. Returns 0 if failed */
double	get_double_f(FILE* input);				/* Gets a double from a file. Returns 0 if failed */

/* Character Functions */
int		is_digit(char c);						/* Gets a character and returns 1 if a digit, 0 if not */
int		is_letter(char c);						/* Gets a character and returns 1 if a letter, 0 if not */
char	lower_char(char c);						/* Gets a character and returns a lower-case char if upper case, unchanged if not */
char	upper_char(char c);						/* Gets a character and returns an upper-case char if lower case, unchanged if not */

/* Single-String Functions */
int		str_length(char* str);					/* Gets a string and returns its length, not including '\0' */
int		str_to_int(char* str);					/* Gets a string and returns its integer value. Returns 0 if failed */
char*	int_to_str(int num);					/* Gets an integer and returns a string that contains that integer. Returns NULL if failed */
void	lower_str(char* str);					/* Gets a string and turns it into an all-lower-case string */
void	upper_str(char* str);					/* Gets a string and turns it into an all-upper-case string */
void	capitalize(char* str);					/* Gets a string and capitalizes the first char and turns into lower-case the rest */
void	str_copy(char* dest, char* source);		/* Gets a string and copies its content to another string. */

/* Two-Strings Functions */
int		substring(char *str1, char *str2);		/* Gets two strings and returns 1 if str2 is a substring of str1. 0 if not */
int		str_compare(char* str1, char* str2);	/* Gets two strings and returns 1 if str1 is greater, 0 if equal, -1 if str2 is greater */
char*	str_connect(char* str1, char* str2);	/* Gets two strings and connects them, separated by a space. Returns NULL if fails */

#endif